
/*	File			BullyElectionAlgorithmDlg.h
	Purpose			Implementation of the Bully Election Algorithm
	Author			Richard Anthony	(R.J.Anthony@gre.ac.uk)
	Date			January 2015
*/

#pragma once
#include "afxwin.h"

#define BULLY_ELECTION_ALGORITHM_PORT		8005	// All messages are received ion this port, regardless of the processes election state
#define MASTER_HEARTBEAT_TX_PERIOD			5000	// Heartbeat transmission period (milliseconds)
#define ELECTION_TRIGGER_HEARTBEATS_MISSED		3	// Threshold number of heartbeats a Slave must miss before initiating an election
#define CHECK_FOR_MESSAGE_PERIOD			100		// Period between checking for any incomming messages (milliseconds)
#define CANDIDATE_STATE_DURATION				3	// The number of heartbeat periods a node must wait in Candidate state before promoting

#define HeartbeatClock_LIMIT				1000

#define TIMER_MASTER_HEARTBEAT_TX			1
#define TIMER_CHECK_FOR_MESSAGE				2

enum BULLY_ELECTION_ALGORITHM_Process_State {State_Slave, State_Candidate, State_Master};
enum BULLY_ELECTION_ALGORITHM_Message_Type {MT_MasterHeartbeat, MT_ElectionTrigger};

struct BULLY_ELECTION_ALGORITHM_Message_Format {
	BULLY_ELECTION_ALGORITHM_Message_Type MessageType;
};

class CBullyElectionAlgorithmDlg : public CDialog
{
	DECLARE_DYNAMIC(CBullyElectionAlgorithmDlg)

public:
	CBullyElectionAlgorithmDlg(CWnd* pParent = NULL);   // standard constructor
	virtual ~CBullyElectionAlgorithmDlg();
	
	
	bool CreateSendSocket();
	void SetUpSendAddressStructForBroadcast();
	IN_ADDR GetBroadcastAddress();

	bool CreateReceiveSocket();
	IN_ADDR GetLocalHostAddressDetails();
	void InitialiseControls();

	void InitiateMasterHeartbeatTimer();
	void InitiateMessageReceiveTimer();

	void InitiateTimer(UINT_PTR piEvent, u_int iMilliSeconds);
	void StopTimer(UINT iEvent);
	void OnTimer(UINT nIDEvent); 
	void CheckPresenceOfMaster(int iCurrentHeartbeatClock);
	
	
	void SendMasterHeartbeatMessage();
	void SendElectionTriggerMessage();
	void Send(BULLY_ELECTION_ALGORITHM_Message_Format Message);
	
	void DoReceive();
	void CompareNodeIDs_DemoteIfRemoteIsHigher(SOCKADDR_IN FromAddr);
	void CompareNodeIDs_PromoteToCandidateIfRemoteIsLower(SOCKADDR_IN FromAddr);
	bool CompareNodeIDs_RemoteIsLower(SOCKADDR_IN FromAddr);

	void WriteEventListEntry(CString csLine);

	
	
	void SetUpLocalAddressStruct();
	void TransitionToMasterState();
	void TransitionToCandidateState();
	void TransitionToSlaveState();

	void ChangeState(BULLY_ELECTION_ALGORITHM_Process_State NewState);
	void DisplayLocalState();
	CString ConvertLocalStateToString();

// Dialog Data
	enum { IDD = IDD_BULLYELECTIONALGORITHM_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnBnClickedCleareventlistButton();
	afx_msg void OnBnClickedDone();

private:
	BULLY_ELECTION_ALGORITHM_Process_State m_LocalState;

		//flags

	// Timer
	UINT_PTR m_nTimer;

	int m_iMasterHeartbeatClock;	// Used to measure time intervals between heartbeats from the Master instance
	int m_iMostRecentHeartbeat_TimeDetected;
	int m_iClock_CountInCandidateState_SincelastChallenge; // Keeps track of how long the node has been in Candidate state
						// Must stay in candidate state long enough to ensure all possible challengers have been defeated


	unsigned short m_iRecvPORT;			// Port on which all messages are received

	SOCKET m_SendSock;		// Socket for all sends (broadcast)
	SOCKET m_RecvSock;		// Socket on which all messsages are received
	
	SOCKADDR_IN m_SendAddr;		// Address for sending all messages (all are broadcast)
	SOCKADDR_IN m_LocalAddr;	// Local Address for receiving all messages
public:
	CListBox* m_pEventList;
	CEdit* m_pCurrentState;
};




