
/*	File			CaseStudyGame_Client_DSDlg.cpp
	Purpose			Network Game (Noughts and Crosses) CLIENT
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			January 2015 (original December 2003)
	
	This version uses the Directory Service: 
		The server registers with the Directory Service,
		Clients (this) use the Directory Service to locate the Server.


	Documentation for the extension of the original game, to use the Directory Service
	**********************************************************************************
	The interface to the directory service is defined as follows:

	Protocol used by the Directory Service: UDP

	Port number allocations
		From_DIRECTORY_SERVICE_PORT	8002	
		The port the application client must listen on for DirectoryServiceReply messages sent in response to RESOLVE request messages.
	
		To_DIRECTORY_SERVICE_PORT	8008
		The port that application clients use to send RESOLVE messages to the directory service (broadcast is used).
		The port that application servers use to send REGISTER messages to the directory service (broadcast is used).

	Message structures
		The message structure returned by the directory service in response to RESOLVE request messages is shown below:
 
		struct DirectoryServiceReply {
			// IP address followed by port
			unsigned char a1;
			unsigned char a2;
			unsigned char a3;
			unsigned char a4;
			PORT port;
			};

	Message formats
		Client RESOLVE request messages are encoded as a character string containing RESOLVE:server_name where server_name 
		can be a maximum of 30 characters.
		An example is      	RESOLVE:GameServer

		Server REGISTER messages are encoded as a character string containing "REGISTER:server_name:port" where server_name 
		can be a maximum of 30 characters, and the port number is expressed in text form.
		An example is 	REGISTER:GameServer:8004

		Note that for both message types the IP address of the sending process is extracted by the directory service, directly
		from the header of the received message; the game component does not have to explicitly send it.
*/

#include "stdafx.h"
#include "CaseStudyGame_Client_DSDlg.h"
#include <sys/types.h>
#include <sys/timeb.h>
#pragma warning(disable : 4996)

IMPLEMENT_DYNAMIC(CaseStudyGame_Client_DSDlg, CDialog)
CaseStudyGame_Client_DSDlg::CaseStudyGame_Client_DSDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CaseStudyGame_Client_DSDlg::IDD, pParent)
{		
	bool bResult = CreateResolveSocket();  // Create the socket for communication with the DS
	if(false == bResult)
	{
		OnBnClickedTcpAoDone();
	}
	
	SetUpLocalAddressStruct(m_LocalAddr, From_DIRECTORY_SERVICE_PORT);
	int iResult = bind(m_ResolveSock, (const SOCKADDR FAR*)&m_LocalAddr, sizeof(m_LocalAddr));
	if(SOCKET_ERROR == iResult)
	{		
		OnBnClickedTcpAoDone();	// Error binding, close dialog
	}
	SetUpSendAddressStructForBroadcast(m_ResolveAddr, To_DIRECTORY_SERVICE_PORT);
}

CaseStudyGame_Client_DSDlg::~CaseStudyGame_Client_DSDlg()
{
	CloseConnection();
}

BEGIN_MESSAGE_MAP(CaseStudyGame_Client_DSDlg, CDialog)
	ON_WM_TIMER()
	ON_BN_CLICKED(ID_DONE, OnBnClickedTcpAoDone)
	ON_BN_CLICKED(IDC_SET_ALIAS, OnBnClickedSetAlias)
	ON_BN_CLICKED(IDC_SELECT_PLAYER, OnBnClickedSelectPlayer)
	ON_BN_CLICKED(IDC_CELL3, OnBnClickedCell3)
	ON_BN_CLICKED(IDC_CELL4, OnBnClickedCell4)
	ON_BN_CLICKED(IDC_CELL5, OnBnClickedCell5)
	ON_BN_CLICKED(IDC_CELL6, OnBnClickedCell6)
	ON_BN_CLICKED(IDC_CELL7, OnBnClickedCell7)
	ON_BN_CLICKED(IDC_CELL8, OnBnClickedCell8)
	ON_BN_CLICKED(IDC_CELL0, OnBnClickedCell0)
	ON_BN_CLICKED(IDC_CELL1, OnBnClickedCell1)
	ON_BN_CLICKED(IDC_CELL2, OnBnClickedCell2)
END_MESSAGE_MAP()

void CaseStudyGame_Client_DSDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_ALIAS_NAME, m_AliasName);
	DDX_Control(pDX, IDC_PLAYER_LIST, m_PlayerListBox);
	DDX_Control(pDX, IDC_CELL0, m_Cell0);
	DDX_Control(pDX, IDC_CELL1, m_Cell1);
	DDX_Control(pDX, IDC_CELL2, m_Cell2);
	DDX_Control(pDX, IDC_CELL3, m_Cell3);
	DDX_Control(pDX, IDC_CELL4, m_Cell4);
	DDX_Control(pDX, IDC_CELL5, m_Cell5);
	DDX_Control(pDX, IDC_CELL6, m_Cell6);
	DDX_Control(pDX, IDC_CELL7, m_Cell7);
	DDX_Control(pDX, IDC_CELL8, m_Cell8);
	DDX_Control(pDX, IDC_TOKEN, m_Token);
	DDX_Control(pDX, IDC_STATUS, m_Status);
	DDX_Control(pDX, IDC_OPPONENT_ALIAS, m_OpponentAlias);
	DDX_Control(pDX, IDC_GAME_STATUS, m_GameStatus);
	DDX_Control(pDX, IDC_SELECT_PLAYER, m_SelectPlayer);
	DDX_Control(pDX, IDC_DirectoryService_EDIT, m_DirectoryService_Edit);
	DDX_Control(pDX, IDC_ConnectionStatus_EDIT, m_ConnectionStatus);

	// controls initialisation
	CString csStr;
	m_pConnectAddress_1 = (CEdit*) GetDlgItem(IDC_ADD_1);
	m_pConnectAddress_2 = (CEdit*) GetDlgItem(IDC_ADD_2);
	m_pConnectAddress_3 = (CEdit*) GetDlgItem(IDC_ADD_3);
	m_pConnectAddress_4 = (CEdit*) GetDlgItem(IDC_ADD_4);
	m_pPort = (CEdit*) GetDlgItem(IDC_PORT);
	m_pRegisterAlias = (CButton*) GetDlgItem(IDC_SET_ALIAS);

	InitialiseControls();
	Contact_DirectoryService();
}

void CaseStudyGame_Client_DSDlg::Contact_DirectoryService()
{
	SendResolveMessage(); // First DS request sent (further requests sent from Timer handler if no reply received)
	m_nTimer = SetTimer((UINT_PTR) 2, 3000, 0);;	// Wait for reply from Directory Service
}

bool CaseStudyGame_Client_DSDlg::CreateResolveSocket()
{   // UDP, non-blocking, broadcast
	m_ResolveSock = socket(AF_INET, SOCK_DGRAM, PF_UNSPEC);
	if(INVALID_SOCKET == m_ResolveSock)
	{
		return false;	// Error creating socket
	}

	unsigned long lNonBlocking = 1;
	int iError = ioctlsocket(m_ResolveSock, FIONBIO, &lNonBlocking);
	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting non-blocking behaviour
	}

	// Turn on Broadcast mode
	char cOpt[2];
	cOpt[0] = 1; // true
	cOpt[1] = 0; // null terminate the option array
	iError = setsockopt(m_ResolveSock, SOL_SOCKET, SO_BROADCAST, cOpt, sizeof(cOpt)); 
  	if(SOCKET_ERROR == iError)
	{
		return false;	// Error setting broadcast option
	}
	return true;
}

void CaseStudyGame_Client_DSDlg::SetUpSendAddressStructForBroadcast(SOCKADDR_IN &Sockaddr_In, unsigned short port)
{	
	Sockaddr_In.sin_addr = GetBroadcastAddress();
	Sockaddr_In.sin_family = AF_INET;
	Sockaddr_In.sin_port = htons(port);
}

IN_ADDR CaseStudyGame_Client_DSDlg::GetBroadcastAddress()
{	
	IN_ADDR addr;
	addr.S_un.S_un_b.s_b1 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b2 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b3 = (unsigned  char) 255;
	addr.S_un.S_un_b.s_b4 = (unsigned  char) 255;
	return addr;
}

void CaseStudyGame_Client_DSDlg::InitialiseControls()
{	
	m_pPort->EnableWindow(false);
	m_pConnectAddress_1->EnableWindow(false);
	m_pConnectAddress_2->EnableWindow(false);
	m_pConnectAddress_3->EnableWindow(false);
	m_pConnectAddress_4->EnableWindow(false);
	m_pRegisterAlias->EnableWindow(false);
	m_AliasName.EnableWindow(false);	
	m_SelectPlayer.EnableWindow(false);
	m_PlayerListBox.EnableWindow(false);
	m_GameStatus.SetWindowText("No active game"); // Display game status
	ClearAllCells();
	m_bSockCreated = false;
	m_bConnected = false;
	m_Token.SetWindowText("");
	m_Status.SetWindowText("");
	m_OpponentAlias.SetWindowText("");	
	m_DirectoryService_Edit.SetWindowText("");
	m_ConnectionStatus.SetWindowText("NOT connected\r\nto Game Server");
}

IN_ADDR CaseStudyGame_Client_DSDlg::GetLocalHostAddressDetails()
{
	char szStr[80];
	DWORD lLen = sizeof(szStr);
    GetComputerNameA(szStr,&lLen);
	hostent* pHost;
	pHost = gethostbyname(szStr);
		
	IN_ADDR addr;
	char*  pChar;
	char** ppChar = pHost->h_addr_list;

	if(ppChar != NULL && *ppChar != NULL)  
	{
		pChar = *ppChar; 
		addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
	}
	else
	{
		MessageBox("Failed to get local address!","Client");
		this->OnCancel();
	}
	return addr;
}

void CaseStudyGame_Client_DSDlg::SetUpLocalAddressStruct(SOCKADDR_IN &Sockaddr_In, unsigned short port)
{	// Initialise the IP address and Port number
	Sockaddr_In.sin_addr = GetLocalHostAddressDetails();
	Sockaddr_In.sin_family = AF_INET;
	Sockaddr_In.sin_port = htons(port);
}

void CaseStudyGame_Client_DSDlg::SendResolveMessage() // Broadcast message to Directory server
{
	char cBuffer[50];
	strcpy_s(cBuffer,"RESOLVE:");
	strcat_s(cBuffer,GAME_SERVER_NAME);
	int iBytesSent;

	iBytesSent = sendto(m_ResolveSock, (char *) cBuffer, sizeof(cBuffer), 0,
							(const sockaddr*)&m_ResolveAddr, sizeof(sockaddr));
	if(SOCKET_ERROR == iBytesSent)
	{
		//iError = WSAGetLastError();
	}
	m_DirectoryService_Edit.SetWindowText("Request sent to DS");
}

void CaseStudyGame_Client_DSDlg::SetUpSendAddress_to_GameServer(DirectoryServiceReply dsr)
{	// The Game Server address and port is retrived from the Directory Service
	m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1 = dsr.a1;
	m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2 = dsr.a2;
	m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3 = dsr.a3;
	m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4 = dsr.a4;
	m_ConnectSockAddr.sin_family = AF_INET;
	m_ConnectSockAddr.sin_port = dsr.iPort;
}

void CaseStudyGame_Client_DSDlg::DisplayConnectAddressAndPort(DirectoryServiceReply dsr)
{	
	CString csStr;
	csStr.Format("%d", m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b1);
	m_pConnectAddress_1->SetWindowText(csStr);
	csStr.Format("%d", m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b2);
	m_pConnectAddress_2->SetWindowText(csStr);
	csStr.Format("%d", m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b3);
	m_pConnectAddress_3->SetWindowText(csStr);
	csStr.Format("%d", m_ConnectSockAddr.sin_addr.S_un.S_un_b.s_b4);
	m_pConnectAddress_4->SetWindowText(csStr);
	csStr.Format("%d",m_ConnectSockAddr.sin_port);
	m_pPort->SetWindowText(csStr.GetString());
}

void CaseStudyGame_Client_DSDlg::ConnectToGameServer()
{	
	// 1. Create socket
	m_iConnectSocket = socket(AF_INET, SOCK_STREAM, PF_UNSPEC); // Create the socket
	if(INVALID_SOCKET == m_iConnectSocket)
	{
		MessageBox("Socket create failed","CNoughtsCrossesCLIENT");
		return;	
	}

	m_bSockCreated = true;

	// 2. Set blocking mode for socket during connection set-up phase
	u_long lBlocking = 0; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
	int iError = ioctlsocket(m_iConnectSocket, FIONBIO, &lBlocking); // Set blocking socket behaviour
	if(NO_ERROR != iError)
	{
		MessageBox("Could not set Blocking behaviour for socket","CNoughtsCrossesCLIENT");
		return;	
	}

	// 3. Setup Connect Address Structure  (this is done directly when the DS returns the address details)
	
	CString csStr;
	m_pPort->GetWindowText(csStr);
	u_short iPort = (u_short) atol(csStr.GetString());
	m_ConnectSockAddr.sin_port = htons(iPort);
	
	// 4. Connect
	iError = connect(m_iConnectSocket, (const SOCKADDR FAR*)&m_ConnectSockAddr, sizeof(m_ConnectSockAddr));
	if(SOCKET_ERROR == iError)
	{
		MessageBox("Connect failed !\n\nIs server address correct ?\nIs port correct ?\nIs server running ?","CNoughtsCrossesCLIENT");
		closesocket(m_iConnectSocket);		// Close the socket
		return;
	}

	m_bConnected = true;
	m_ConnectionStatus.SetWindowText("  CONNECTED\r\nto Game Server");

	// 5. Update controls
	m_pPort->EnableWindow(false);
	m_pConnectAddress_1->EnableWindow(false);
	m_pConnectAddress_2->EnableWindow(false);
	m_pConnectAddress_3->EnableWindow(false);
	m_pConnectAddress_4->EnableWindow(false);
	m_bConnected = true;
	m_AliasName.EnableWindow(true);	
	m_pRegisterAlias->EnableWindow(true);

	// 6. Set non-blocking (now that connection established) for receiving
	lBlocking = 1; // 0 = blocking is enabled, non 0 = non-blocking is enabled.
	iError = ioctlsocket(m_iConnectSocket, FIONBIO, &lBlocking); // Set non-blocking socket behaviour
	if(NO_ERROR != iError)
	{
		MessageBox("Could not set non-Blocking behaviour for socket","CNoughtsCrossesCLIENT");
		return;	
	}

	// 7. Enable receiving 
	DoReceive();
}

bool CaseStudyGame_Client_DSDlg::Send(Message_PDU Message)
{
	int iBytesSent;

	iBytesSent = send(m_iConnectSocket, (char *) &Message, sizeof(Message_PDU), 0);
	if(SOCKET_ERROR == iBytesSent)
	{
		int iError = WSAGetLastError();
		StopTimerIfConnectionLost(iError);
		return false;
	}
	return true;
}

void CaseStudyGame_Client_DSDlg::InitiateTimer()
{	
	m_nTimer = SetTimer(1, 100, 0);	// 100 ms one-shot for UI responsiveness
}

void CaseStudyGame_Client_DSDlg::StopTimer()
{
	KillTimer(m_nTimer);
}

void CaseStudyGame_Client_DSDlg::OnTimer(UINT nIDEvent) 
{	
	CString csStr;

	switch (nIDEvent)
	{
	case 1: // Here to check for Game Messages
        DoReceive();
		break;
	case 2: // Here to check for responses from the Directory Service
		DirectoryServiceReply Message_DirectoryServiceReply; // Create a structure to hold the response from the Directory Service
		int iBytesRecd = recvfrom(m_ResolveSock, (char *)& Message_DirectoryServiceReply, sizeof(DirectoryServiceReply), 0, NULL, NULL);
		if(iBytesRecd > 0)
		{
			if(iBytesRecd == sizeof(DirectoryServiceReply)) // Expected reply length
			{

				if(Message_DirectoryServiceReply.iPort != 0)	// Port value 0 means App Service was not known, ONLY update controls if the reply was valid
				{
					csStr.Format("Received %d.%d.%d.%d:%d", Message_DirectoryServiceReply.a1,
										Message_DirectoryServiceReply.a2, Message_DirectoryServiceReply.a3,
										Message_DirectoryServiceReply.a4, Message_DirectoryServiceReply.iPort);
					m_DirectoryService_Edit.SetWindowText(csStr.GetString());

					KillTimer(2);	// Received a reply from DS, so stop the timer (stop requesting the Game Server address from the DS)
					closesocket(m_ResolveSock); // Close the socket as soon as a valid Directory Service reply received so to not hog the bound port
												// This allows other clients to run at the same host computer - very useful for development and testing purposes
						
					if(false == m_bConnected)
					{
						SetUpSendAddress_to_GameServer(Message_DirectoryServiceReply);
						DisplayConnectAddressAndPort(Message_DirectoryServiceReply);
						ConnectToGameServer(); // Automatically connect to Game Server using address and port details supplied by the Directory Service
					}
				}
			}
		}
		else
		{	// No reply received from Directory Service, resend the request (the timer is left running, to listen for responses from the DS)	
			SendResolveMessage(); // Broadcast message to Directory server
		}
		break;
	}
}

void CaseStudyGame_Client_DSDlg::DoReceive()
{		
	StopTimer();

	int iBytesRecd;
	Message_PDU Message;

	iBytesRecd = recv(m_iConnectSocket, (char *) &Message, sizeof(Message_PDU), 0);
	if(SOCKET_ERROR == iBytesRecd)
	{
		int iError = WSAGetLastError();
		if(WSAEWOULDBLOCK == iError)	// Non-blocking socket used	(if no packet is queued for the socket the operation would otherwise block)

		{
			InitiateTimer();			// 100ms interval
		}
		else
		{
			StopTimerIfConnectionLost(iError);
		}
		return;
	}
	else								// A message was received
	{
		if(iBytesRecd == 0)
		{
			ConnectionLost();
			return;
		}

		switch(Message.iMessageType)
		{
		case PLAYER_LIST:
		  DisplayPlayerList(Message);	
		  break;
		case START_OF_GAME:
			m_SelectPlayer.EnableWindow(false);				// Disable Select Opponent control
			m_OpponentAlias.SetWindowText(Message.cAlias);	// Set and display opponent alias
			if(Message.iToken == 0) 						// Set and display our token
			{
				m_csToken = "0";
				m_csOpponentToken = "X";
			}
			else
			{
				m_csToken = "X";
				m_csOpponentToken = "0";
			}		
			m_Token.SetWindowText(m_csToken.GetString());
			
			if(Message.bFirstMove == true)					// configure first-move logic
			{
				m_Status.SetWindowText("Your Move");
				EnableEmptyCells();
			}
			else
			{
				m_Status.SetWindowText("Opponent's Move");
				DisableEmptyCells();
			}
			m_GameStatus.SetWindowText("Game In Progress"); // Display game status
		  break;
		case END_OF_GAME:
			switch(Message.iCode) 
			{
			case END_OF_GAME_DRAW:
			  MessageBox("The game was a draw","Game Over");	
			  break;
			case END_OF_GAME_WIN:
			  MessageBox("Well done! You won","Game Over");	
			  break;
			case END_OF_GAME_LOOSE:
			  MessageBox("Unlucky - You lost","Game Over");	
			  break;
			}
			OnBnClickedTcpAoDone();	// exit
			break;
		case OPPONENT_CELL_SELECTION:
		  switch (Message.iCell)
			{
		    case 0:
			  m_Cell0.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell0.EnableWindow(false);
			  break;
		    case 1:
			  m_Cell1.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell1.EnableWindow(false);
			  break;
		    case 2:
			  m_Cell2.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell2.EnableWindow(false);
			  break;
		    case 3:
			  m_Cell3.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell3.EnableWindow(false);
			  break;
		    case 4:
			  m_Cell4.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell4.EnableWindow(false);
			  break;
		    case 5:
			  m_Cell5.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell5.EnableWindow(false);
			  break;
		    case 6:
			  m_Cell6.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell6.EnableWindow(false);
			  break;
		    case 7:
			  m_Cell7.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell7.EnableWindow(false);
			  break;
		    case 8:
			  m_Cell8.SetWindowText(m_csOpponentToken.GetString());
			  m_Cell8.EnableWindow(false);
			  break;
			}
			m_Status.SetWindowText("Your Move");
			EnableEmptyCells();
			break;
		}
	}
	InitiateTimer();
}

void CaseStudyGame_Client_DSDlg::DisplayPlayerList(Message_PDU Message)
{
	// clear contents of PlayerList control
	while(m_PlayerListBox.GetCount() > 0)
	{	
		m_PlayerListBox.DeleteString(0);
	}

	CString csAlias;
	m_AliasName.GetWindowText(csAlias); // Read our Alias (exclude it from displayed list)
	
	CString csStr(Message.cPlayerList); // comma-separated, null-terminated list
	int iNextCommaPos;
	CString csCurrentPlayer;
	while((iNextCommaPos = csStr.Find(',')) > 0)
	{
		csCurrentPlayer = csStr.Left(iNextCommaPos);
		if(csCurrentPlayer.CompareNoCase(csAlias) != 0) // Exclude own alias from displayed list
		{
			m_PlayerListBox.AddString(csCurrentPlayer.GetString());
		}
		csStr = csStr.Right(csStr.GetLength() - (iNextCommaPos + 1));
	}
	if(csStr.CompareNoCase(csAlias) != 0) // Exclude own alias from displayed list
	{
		m_PlayerListBox.AddString(csStr.GetString());
	}
	m_PlayerListBox.SetCurSel(0); // Set default selection
}

void CaseStudyGame_Client_DSDlg::StopTimerIfConnectionLost(int iError)			

{
	if(WSAECONNABORTED == iError || WSAECONNRESET == iError ||
		WSAETIMEDOUT == iError || WSAENETRESET == iError ||
		WSAESHUTDOWN == iError)
	{	
		StopTimer();
		this->OnCancel();
	}
}

void CaseStudyGame_Client_DSDlg::ConnectionLost()
{
	OnBnClickedTcpAoDone();
}

void CaseStudyGame_Client_DSDlg::CloseConnection()
{	
	if(m_bConnected)	// if connected - close the connection
	{
		shutdown(m_iConnectSocket,SD_BOTH);	// Close the connection (in both directions)
	}

	if(m_bSockCreated)	// if socket created - close the socket
	{
		closesocket(m_iConnectSocket);	// Close the socket
	}

	closesocket(m_ResolveSock);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedTcpAoDone()
{
	if(m_bConnected)	// If connected - send disconnection message (graceful disconnect)
	{
		Message_PDU Message;
		Message.iMessageType = CLIENT_DISCONNECT;
		Send(Message);
	}

	CloseConnection();
	this->OnCancel();
}

void CaseStudyGame_Client_DSDlg::OnBnClickedSetAlias() // Register Alias with server
{
	CString csStr;
	m_AliasName.GetWindowText(csStr);
	if(csStr.GetLength() == 0)
	{
		MessageBox("Alias name cannot be blank","'Noughts and Crosses' Client");
	}
	else
	{
		Message_PDU Message;
		Message.iMessageType = REGISTER_ALIAS;
		strncpy(Message.cAlias,csStr.GetString(),MAX_ALIAS_LEN);
		if(Send(Message) == true)
		{
			m_AliasName.EnableWindow(false);	
			m_pRegisterAlias->EnableWindow(false);
			m_PlayerListBox.EnableWindow(true);
			m_SelectPlayer.EnableWindow(true);
		}
	}
}

void CaseStudyGame_Client_DSDlg::OnBnClickedSelectPlayer()
{
	int iSelection = m_PlayerListBox.GetCurSel();
	if(-1 == iSelection)
	{
		MessageBox("No player selected","'Noughts and Crosses' Client");
	}
	else
	{
		char cBuffer[MAX_ALIAS_LEN + 1];
		int iLen = m_PlayerListBox.GetText(iSelection,cBuffer);
	
		// Send a CHOOSE_OPPONENT message to server
		Message_PDU Message;
		Message.iMessageType = CHOOSE_OPPONENT;
		strcpy(Message.cAlias,cBuffer);
		Send(Message);
	}
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell0()
{
	m_Cell0.SetWindowText(m_csToken.GetString());
	m_Cell0.EnableWindow(false);
	CellSelectAction(0);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell1()
{
	m_Cell1.SetWindowText(m_csToken.GetString());
	m_Cell1.EnableWindow(false);
	CellSelectAction(1);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell2()
{
	m_Cell2.SetWindowText(m_csToken.GetString());
	m_Cell2.EnableWindow(false);
	CellSelectAction(2);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell3()
{
	m_Cell3.SetWindowText(m_csToken.GetString());
	m_Cell3.EnableWindow(false);
	CellSelectAction(3);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell4()
{
	m_Cell4.SetWindowText(m_csToken.GetString());
	m_Cell4.EnableWindow(false);
	CellSelectAction(4);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell5()
{
	m_Cell5.SetWindowText(m_csToken.GetString());
	m_Cell5.EnableWindow(false);
	CellSelectAction(5);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell6()
{
	m_Cell6.SetWindowText(m_csToken.GetString());
	m_Cell6.EnableWindow(false);
	CellSelectAction(6);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell7()
{
	m_Cell7.SetWindowText(m_csToken.GetString());
	m_Cell7.EnableWindow(false);
	CellSelectAction(7);
}

void CaseStudyGame_Client_DSDlg::OnBnClickedCell8()
{
	m_Cell8.SetWindowText(m_csToken.GetString());
	m_Cell8.EnableWindow(false);
	CellSelectAction(8);
}

void CaseStudyGame_Client_DSDlg::CellSelectAction(int iCell)
{
	// send a CELL_SELECTION message to server
	Message_PDU Message;
	Message.iMessageType = LOCAL_CELL_SELECTION;
	Message.iCell = iCell;
	if(Send(Message) == true)
	{
		m_Status.SetWindowText("Opponent's Move");
		DisableEmptyCells();
	}
}

void CaseStudyGame_Client_DSDlg::DisableEmptyCells()
{
	CString csStr;
	m_Cell0.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell0.EnableWindow(false); 
	m_Cell1.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell1.EnableWindow(false); 
	m_Cell2.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell2.EnableWindow(false); 
	m_Cell3.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell3.EnableWindow(false); 
	m_Cell4.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell4.EnableWindow(false); 
	m_Cell5.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell5.EnableWindow(false); 
	m_Cell6.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell6.EnableWindow(false); 
	m_Cell7.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell7.EnableWindow(false); 
	m_Cell8.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell8.EnableWindow(false); 
}

void CaseStudyGame_Client_DSDlg::EnableEmptyCells()
{
	CString csStr;
	m_Cell0.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell0.EnableWindow(true); 
	m_Cell1.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell1.EnableWindow(true); 
	m_Cell2.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell2.EnableWindow(true); 
	m_Cell3.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell3.EnableWindow(true); 
	m_Cell4.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell4.EnableWindow(true); 
	m_Cell5.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell5.EnableWindow(true); 
	m_Cell6.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell6.EnableWindow(true); 
	m_Cell7.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell7.EnableWindow(true); 
	m_Cell8.GetWindowText(csStr);
	if(csStr.IsEmpty()) m_Cell8.EnableWindow(true); 
}

void CaseStudyGame_Client_DSDlg::ClearAllCells()
{
	m_Cell0.SetWindowText("");
	m_Cell1.SetWindowText("");
	m_Cell2.SetWindowText("");
	m_Cell3.SetWindowText("");
	m_Cell4.SetWindowText("");
	m_Cell5.SetWindowText("");
	m_Cell6.SetWindowText("");
	m_Cell7.SetWindowText("");
	m_Cell8.SetWindowText("");
	DisableEmptyCells();
}
