
/*	File			CaseStudyGame_Client_DSDlg.h
	Purpose			Network Game (Noughts and Crosses) CLIENT
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			April 2014 (original December 2003)
	
	This version uses the Directory Service to locate the Game Server
*/
#pragma once
#include "CaseStudyGame_Client_DS.h"
#include "NoughtsCrosses_General.h"
#include "afxwin.h"

class CaseStudyGame_Client_DSDlg : public CDialog
{
	DECLARE_DYNAMIC(CaseStudyGame_Client_DSDlg)

public:
	CaseStudyGame_Client_DSDlg(CWnd* pParent = NULL);
	virtual ~CaseStudyGame_Client_DSDlg();
	void Contact_DirectoryService();
	bool CreateResolveSocket();
	void SetUpSendAddressStructForBroadcast(SOCKADDR_IN &Sockaddr_In, unsigned short port);
	IN_ADDR GetBroadcastAddress();
	void InitialiseControls();
	IN_ADDR GetLocalHostAddressDetails();
	void SetUpLocalAddressStruct(SOCKADDR_IN &Sockaddr_In, unsigned short port);
	void SendResolveMessage();
	void SetUpSendAddress_to_GameServer(DirectoryServiceReply dsr);
	void DisplayConnectAddressAndPort(DirectoryServiceReply dsr);
	void ConnectToGameServer();
	void OnBnClickedEnableReceiver();
	void InitiateTimer();
	void StopTimer();
	void OnTimer(UINT nIDEvent); 
	void DoReceive();
	void CloseConnection();
	void StopTimerIfConnectionLost(int iError);
	void ConnectionLost();
	void DisplayPlayerList(Message_PDU Message);
	void ClearAllCells();
	void CellSelectAction(int iCell);
	void DisableEmptyCells();
	void EnableEmptyCells();
	bool Send(Message_PDU Message);

	enum { IDD = IDD_CASESTUDYGAME_CLIENT_DS_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()
private:
	// Communications
	SOCKET m_iConnectSocket;
	SOCKET m_ResolveSock;			// UDP Socket on which 'Resolve' messages are sent and replies received
	int m_iSendLen;
	SOCKADDR_IN m_ConnectSockAddr;
	SOCKADDR_IN m_ResolveAddr;		// For Broadcasting 'resolve' messages to the Directory Server
	SOCKADDR_IN m_LocalAddr;		// Address for Receiving responses from DS
	
	//flags
	bool m_bSockCreated;
	bool m_bConnected;
	// Timer
	UINT_PTR m_nTimer;
	int m_iDelay;

public:
	afx_msg void OnBnClickedTcpAoDone();
	afx_msg void OnBnClickedSetAlias();
	afx_msg void OnBnClickedSelectPlayer();
	afx_msg void OnBnClickedCell0();
	afx_msg void OnBnClickedCell1();
	afx_msg void OnBnClickedCell2();
	afx_msg void OnBnClickedCell3();
	afx_msg void OnBnClickedCell4();
	afx_msg void OnBnClickedCell5();
	afx_msg void OnBnClickedCell6();
	afx_msg void OnBnClickedCell7();
	afx_msg void OnBnClickedCell8();

	CListBox m_PlayerListBox;
	CEdit* m_pConnectAddress_1;
	CEdit* m_pConnectAddress_2;
	CEdit* m_pConnectAddress_3;
	CEdit* m_pConnectAddress_4;
	CEdit* m_pPort;
	CEdit* m_pSocketNumber;
	CEdit* m_pStatus;
	CEdit m_AliasName;
	CEdit m_Token;
	CEdit m_Status;
	CEdit m_OpponentAlias;
	CEdit m_GameStatus;
	CButton* m_pRegisterAlias;
	CButton m_SelectPlayer;
	CButton m_Cell0;
	CButton m_Cell1;
	CButton m_Cell2;
	CButton m_Cell3;
	CButton m_Cell4;
	CButton m_Cell5;
	CButton m_Cell6;
	CButton m_Cell7;
	CButton m_Cell8;
	CString m_csToken;
	CString m_csOpponentToken;
	CEdit m_DirectoryService_Edit;
	CEdit m_ConnectionStatus;
};
