/*	File			CaseStudyGame_Server_DSDlg.h
	Purpose			Network Game (Noughts and Crosses) SERVER
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			January 2015

	This version uses the Directory Service:
		The server (this) registers with the Directory Service,
		Clients use the Directory service to locate the Server.
*/

#pragma once
#include "NoughtsCrosses_General.h"
#include "afxwin.h"

#define MAX_GAMES 5
#define MAX_CONNECTIONS MAX_GAMES * 2

struct Connection {
	SOCKET m_iConnectSock;
	SOCKADDR_IN m_ConnectSockAddr;
	CString csAlias;
	bool bInUse;
};

struct Game {
	int iConnection[2];
	bool bInUse;
	int iGrid[3][3];
};

class CCaseStudyGame_Server_DSDlg : public CDialog
{
	DECLARE_DYNAMIC(CCaseStudyGame_Server_DSDlg)

public:
	CCaseStudyGame_Server_DSDlg(CWnd* pParent = NULL);   // standard constructor
	virtual ~CCaseStudyGame_Server_DSDlg();
	void InitialiseControls();
	IN_ADDR GetLocalHostAddressDetails();
	void InitialiseLocalAddressStructAndPort();
	void ReadFinalPortValue();
	void InitiateTimer();					// REPEAT for non-blocking recv's
	void InitiateRegistrationTimer();
	void StopTimer();
	void OnTimer(UINT nIDEvent); 
	void DoReceive(int iConnection);
	void CloseAll();
	void WriteStatusLine(CString csLine);
	int GetIndexOfUnusedConnection();
	int GetIndexOfUnusedGame();
	int CountActiveConnections();
	int CountActiveGames();
	void ConnectionLost(int iConnection, int iError);			
	void CloseConnection(int iConnection);
	void SendPlayerList();
	bool CreateGame(int iConnection, Message_PDU Message);
	bool IsConnectionPartOfActiveGame(int iConnection);
	int FindIndexOfConnectionForAlias(CString csAlias);
	void SendOpponentCellSelectionMessage(int iConnection, int iCell);
	int GetConnectionOpponent(int iConnection);
	int FindGameForConnection(int iConnection);	
	int GetPlayerToken(int iGame, int iConnection);		
	void DoSetCellInGame(int iGame, int iCell, int iToken);	
	void SetCellInGame(int iConnection, int iCell);
	bool CheckForWin(int iConnection);
	bool EndOfGame(int iConnection);
	bool CheckForLine(int iGame, int iToken);
	void RemoveGameAndConnections(int iGame);
	void SeedRandomNumberGenerator();
	bool CreateRandomBool();
	void CloseGameAndConnection(int iConnection);
	void Accept();
	void Send(int iConnection, Message_PDU Message);
	void MainLoop();
	bool CreateRegisterSocket();
	void SetUpSendAddressStructForBroadcast(SOCKADDR_IN &Sockaddr_In, unsigned short port);
	IN_ADDR GetBroadcastAddress();
	void SendRegisterMessage(); // Broadcast message to Directory Server

	enum { IDD = IDD_CASESTUDYGAME_SERVER_DS_DIALOG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()
private:
	CString m_csAppName;
	bool bRegisterAlreadyDone; // Keep track of whether the Game Server has registered with the Directory Service

	CEdit* m_pListenPort;
	CEdit* m_pLocalAddress;
	CEdit* m_pStatus;
	CEdit m_ActiveGamesCount;
	CEdit m_ConnectedCount;
	CButton* m_pEnableServer;
	
	SOCKET m_RegisterSock;		// Socket for sending 'Register' messages to Directory Service
	SOCKET m_iListenSock;

	SOCKADDR_IN m_LocalSockAddr;
	SOCKADDR_IN m_RegisterAddr;	// for Broadcasting 'register' messages to the Directory server

	Connection m_ConnectionArray[MAX_CONNECTIONS];
	Game m_GameArray[MAX_GAMES];

	//flags
	bool m_bServerEnabled;

	// Timer
	UINT_PTR m_nTimer;
public:
	afx_msg void OnBnClickedEnableServer();
	afx_msg void OnBnClickedClose();
	afx_msg void OnBnClickedClearEventList();
};

