
/*	File			DNS_ResolverDlg.cpp
	Purpose			Demonstration of the use of the gethostbyname DS resolver
	Author			Richard Anthony	(ar26@gre.ac.uk)
	Date			August 2014
*/

#include "stdafx.h"
#include "DNS_Resolver.h"
#include "DNS_ResolverDlg.h"
#include "afxdialogex.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CDNS_ResolverDlg::CDNS_ResolverDlg(CWnd* pParent /*=NULL*/)
	: CDialogEx(CDNS_ResolverDlg::IDD, pParent)
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CDNS_ResolverDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialogEx::DoDataExchange(pDX);
	m_pHostName= (CEdit*) GetDlgItem(IDC_HostName);

	m_pAddressType = (CEdit*) GetDlgItem(IDC_AddressType);
	m_pHostAddressList = (CListBox*) GetDlgItem(IDC_HOST_ADDRESS_LIST);
	m_pLookupHostName = (CEdit*) GetDlgItem(IDC_LookupHostName);
	m_pFoundIPAddress = (CEdit*) GetDlgItem(IDC_Found_IPAddress);

	GetLocalHostAddress_List();
}

BEGIN_MESSAGE_MAP(CDNS_ResolverDlg, CDialogEx)
	ON_BN_CLICKED(IDC_RESOLVE_BUTTON, &CDNS_ResolverDlg::OnBnClickedResolveButton)
END_MESSAGE_MAP()

BOOL CDNS_ResolverDlg::OnInitDialog()
{
	CDialogEx::OnInitDialog();
	return TRUE;  // return TRUE  unless you set the focus to a control
}

void CDNS_ResolverDlg::GetLocalHostAddress_List()
{	
	char szStr[80];
	DWORD lLen = sizeof(szStr);
	GetComputerNameA(szStr,&lLen);
	hostent* pHost;
	pHost = gethostbyname(szStr);

	m_pHostName->SetWindowText(pHost->h_name);	// Display host name

	CString csStr = "Other";					// Display address type as string
	switch(pHost->h_addrtype)
	{
		case AF_UNSPEC:
			csStr = "AF_UNSPEC";
			break;
		case AF_UNIX:
			csStr = "AF_UNIX";
			break;
		case AF_INET:
			csStr = "AF_INET";
			break;
	}
	m_pAddressType->SetWindowText(csStr);

	int iAddressCount = 0;
	IN_ADDR addr;
	char **ppChar = pHost->h_addr_list;	// Initialise outer pointer (point to 1st byte of 1st address in list)
	char * pChar;						// Inner pointer (addresses do not follow on in sequential memory after each other)
										// Each address is in a separate memory area, so cannot simply increment the inner 
										// pointer to next address, after end of current address. Instead, increment outer
										// pointer, then re-initialise inner pointer to 1st byte of new address

	while(ppChar != NULL && *ppChar != NULL && iAddressCount < MAX_NUM_IP_ADDRESSES) 
	{
		pChar = *ppChar;				// (re) Initialise inner pointer to start of current address
		addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
		addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
		ppChar++;						// Advance outer pointer, point to next IP address (or NULL if no more addresses)
	
		// Display the local address value
		csStr.Format("%d.%d.%d.%d", addr.S_un.S_un_b.s_b1, addr.S_un.S_un_b.s_b2, 
									addr.S_un.S_un_b.s_b3, addr.S_un.S_un_b.s_b4);
		WriteStatusLine(csStr);
	}
}

void CDNS_ResolverDlg::OnBnClickedResolveButton()
{
	CString csStr;
	m_pLookupHostName->GetWindowText(csStr);	// Get the textual name provided by the user
 	hostent* pHost;
	pHost = gethostbyname(csStr.GetString());	// Use gethostbyname to populate the hostent structure

	if(NULL != pHost)
	{
		IN_ADDR addr;
		char*  pChar;
		char** ppChar = pHost->h_addr_list;

		if(ppChar != NULL && *ppChar != NULL)	// If the address list contains at least one IP address, show first address
		{
			pChar = *ppChar; 

			addr.S_un.S_un_b.s_b1 = (unsigned  char) *pChar++;
			addr.S_un.S_un_b.s_b2 = (unsigned  char) *pChar++;
			addr.S_un.S_un_b.s_b3 = (unsigned  char) *pChar++;
			addr.S_un.S_un_b.s_b4 = (unsigned  char) *pChar;
			csStr.Format("%d.%d.%d.%d", addr.S_un.S_un_b.s_b1, addr.S_un.S_un_b.s_b2, 
										addr.S_un.S_un_b.s_b3, addr.S_un.S_un_b.s_b4);
		}
		else									// Report various errors, getting address
		{
			csStr = "Failed to get local address!";	 
		}
	}
	else
	{	
		int iError = WSAGetLastError();
		switch (iError) {
			case WSAHOST_NOT_FOUND:
				csStr = "Authoritative host not found";
				break;
			case WSATRY_AGAIN:
				csStr = "Nonauthoritative host not found / Server failure";
				break;
			case WSANO_RECOVERY:
				csStr = "Nonrecoverable error";
				break;
			case WSANO_DATA:
				csStr = "Valid name, no data of requested type";
				break;
			default: 
				csStr.Format("gethostbyname error %d",iError);
		} 
	}		
	m_pFoundIPAddress->SetWindowText(csStr.GetString());
}

void CDNS_ResolverDlg::WriteStatusLine(CString csLine)
{
	int iCount = m_pHostAddressList->GetCount();
	m_pHostAddressList->InsertString(iCount, csLine);
}



